import actor

class Enemy:
    def __init__(self, eType, x, y):
        self.actor = actor.Actor()
        self.eType = eType
        if self.eType == "Robot":
            self.actor.loadSprite("dummy_stand1.png","stand1")
            self.actor.setPos(x,y)
            self.life = 50 # robot's default life is 100
            self.movingLeft = True # are we scrolling left?
        self.actor.setSprite("stand1")
        self.width = self.actor.rect.width
        self.height = self.actor.rect.height
        self.halfwidth = self.width/2
        self.halfheight = self.height/2
        self.upsideDown = False

    def update(self, curLevel, rx, ry):
        if self.eType == "Robot":
            self.robotUpdate(curLevel,rx,ry)

    def robotUpdate(self, curLevel, rx, ry):
        if self.eType == "Robot":
            # If our robot is too far away from the enemy, skip the AI update
            if ( abs(rx-self.actor.px) > 800 ):
                return
            if self.actor.inFlight() == False:
                if self.movingLeft == True:
                    self.actor.setVelocityX(-2.0)
                elif self.movingLeft == False:
                    self.actor.setVelocityX(2.0)
            else:
                self.actor.setVelocityX(0.0)
            self.actor.update()
            # Physics & AI?
            # Collision detection for feet
            colX = {}
            colY = {}

            actorX,actorY = self.actor.getPos()
            # we know that the robots width & height are 32!
            colX['bm'] = int(actorX + (self.halfwidth))
            colY['bm'] = int(actorY + (self.height))
            colX['tm'] = int(actorX + (self.halfwidth))
            colY['tm'] = int(actorY)
            colX['lm'] = int(actorX)
            colY['lm'] = int(actorY + (self.halfheight))
            colX['rm'] = int(actorX + (self.width))
            colY['rm'] = int(actorY + (self.halfheight))

            collisionOrder = ['lm','rm','tm','bm']

            for colName in collisionOrder:
                ty = (colY[colName]/32) * curLevel.getWidth()
                tx = (colX[colName]/32)
                if self.actor.inFlight() == True:
                    tile = curLevel.data[3][(ty+tx)]
                    if ( tile != 0 ): # 0 is universal empty?
                        # make this universal?
                        if ( colName == 'bm' ):
                            if self.actor.upsideDown == False:
                                self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                                self.actor.hitFloor()
                            else:
                                self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                                self.actor.setVelocityY(0.0)
                        elif ( colName == 'tm' ):
                            if self.actor.upsideDown == False:
                                self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                                self.actor.setVelocityY(0.0)
                            else:
                                self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                                # -1 because we are technically moving the y down 1 more (y+32 when y is zero based!)
                                self.actor.hitFloor()
                        elif ( colName == 'lm' ):
                            self.actor.px = self.actor.px + (32-(colX[colName] % 32))-1
                        elif ( colName == 'rm' ):
                            self.actor.px = self.actor.px - (colX[colName] % 32)
                elif self.actor.inFlight() == False:
                    if ( curLevel.data[3][(ty+tx)] == 0 ): # 0 is universal empty?
                        if self.actor.upsideDown == False:
                            if ( colName == 'bm' ):
                                #self.actor.fall()
                                self.movingLeft = not self.movingLeft
                        else:
                            if ( colName == 'tm' ):
                                #self.actor.fall()
                                self.movingLeft = not self.movingLeft
                    elif ( curLevel.data[3][(ty+tx)] != 0 ): # tile is not empty
                        if ( colName == 'lm' ):
                            self.actor.px = self.actor.px + (32-(colX[colName] % 32))
                            self.movingLeft = not self.movingLeft
                        elif ( colName == 'rm' ):
                            self.actor.px = self.actor.px - (colX[colName] % 32)
                            self.movingLeft = not self.movingLeft


    def draw(self, screen, camX, camY):
        self.actor.draw(screen, camX, camY, not self.movingLeft, self.actor.upsideDown)

    def takeDamage(self, damage, selfList):
        self.life -= damage
        if self.life <= 0:
            selfList.remove(self)
