import actor,data, math,pygame,fx

class Projectile:
    def __init__(self, name, x, y, vx, vy, target=None):
        self.actor = actor.Actor()
        if name == "Climb":
            self.actor.loadSprite("climber_projectile.png","inflight")
            self.actor.setSprite("inflight")
            self.explodeSound = data.load_sound('explode1.wav')
            self.damage = 20
            self.lifeSpan = 500
            self.startX = x
            self.startY = y
            self.boomAngle = 0 # only used for boomerang!
            self.rotateSpeed = 20
            if vx > 0:
                self.actor.setAccelX(7.0) # initial throw
            elif vx < 0:
                self.actor.setAccelX(-7.0) # initial throw
        elif name == "Gravity":
            self.actor.loadSprite("gravity_projectile.png","inflight")
            self.actor.setSprite("inflight")
            self.explodeSound = data.load_sound('explode2.wav')
            self.targetPtr = target
            self.damage = 10
            self.lifeSpan = 300
        elif name == "Runner":
            self.actor.loadSprite("runner_projectile.png","inflight")
            self.actor.setSprite("inflight")
            self.explodeSound = data.load_sound('explode3.wav')
            self.damage = 50
            self.lifeSpan = 15
        elif name == "Firepower":
            self.actor.loadSprite("firepower_projectile.png","inflight")
            self.actor.setSprite("inflight")
            self.explodeSound = data.load_sound('explode4.wav')
            self.damage = 50
            self.target = target
            self.smokeCnt = 0
            self.smokeMax = 5
            self.lifeSpan = 200
        self.pType = name
        self.actor.setPos(x,y)
        if ( vx < 0):
            self.facingLeft = True
        elif ( vx >= 0 ):
            self.facingLeft = False
        self.actor.setVelocityX(vx)
        self.actor.setVelocityY(vy)
        self.actor.gravity = 0.0 # no gravity!
        self.halfwidth = self.actor.rect.width/2
        self.halfheight = self.actor.rect.height/2
        self.width = self.actor.rect.width
        self.height = self.actor.rect.height

    def update(self, curLevel, rx, ry, selfList, fxList=None):
        if ( self.pType == "Firepower" ):
            if self.facingLeft == True:
                self.actor.setAccelX(-0.1)
            else:
                self.actor.setAccelX(0.1) # special case for firepower rockets
            if self.smokeCnt < self.smokeMax:
                self.smokeCnt+=1
            else:
                self.smokeCnt = 0
                fxList.append(fx.FX("Smoke",self.actor.px,self.actor.py))
            if self.target != None:
                angle = math.atan2((self.target.actor.py-self.actor.py),(self.target.actor.px-self.actor.px))
                if self.facingLeft == True:
                    self.actor.vx -= 0.1 * math.cos(angle)
                else:
                    self.actor.vx += 0.1 * math.cos(angle)
                self.actor.vy += 0.3 * math.sin(angle)
                rotAngle = int(360-angle*180/3.141592)
                if rotAngle < 0:
                    rotAngle += 360
                elif rotAngle > 360:
                    rotAngle -= 360
                self.actor.currentImage = pygame.transform.rotate(self.actor._currentImage, rotAngle)
        elif ( self.pType == "Climb"):
               self.boomAngle += self.rotateSpeed
               if self.boomAngle >= 360:
                   self.boomAngle = 0
               center = self.actor.rect.center
               self.actor.currentImage = pygame.transform.rotate(self.actor._currentImage, self.boomAngle)
               self.actor.rect = self.actor.currentImage.get_rect(center=center)
               # do boomerang physics here!
               diff = self.actor.px-self.startX
               if diff < 0: # initial negative
                   self.actor.setAccelX(0.1)
               elif diff > 0:
                   self.actor.setAccelX(-0.1)
                #else if 0, we started, so don't affect the accel
               
        self.lifeSpan -= 1
        self.actor.update()
        actorX,actorY = self.actor.getPos()
        if ( abs(actorX-rx) > 640 ) or ( abs(actorY-ry) > 240 ) or ( actorX <= 0 ) or ( actorY <= 0 ) or ( self.lifeSpan <= 0 ):
            if self.pType == "Firepower":
                fxList.append(fx.FX("Explosion1",self.actor.px,self.actor.py))
                self.explode(selfList)
            else:
                if selfList.count(self) > 0:
                    selfList.remove(self)
            return
        colX = {}
        colY = {}
        # we know that the robots width & height are 32!
        colX['bm'] = int(actorX + self.halfwidth)
        colY['bm'] = int(actorY + self.height)
        colX['tm'] = int(actorX + self.halfheight)
        colY['tm'] = int(actorY)
        colX['lm'] = int(actorX)
        colY['lm'] = int(actorY + self.halfheight)
        colX['rm'] = int(actorX + self.width)
        colY['rm'] = int(actorY + self.halfheight)
        collisionOrder = ['lm','rm','tm','bm']
        for colName in collisionOrder:
            ty = (colY[colName]/32) * curLevel.getWidth()
            tx = (colX[colName]/32)
            if self.actor.inFlight() == True:
                tile = curLevel.data[3][(ty+tx)]
                if ( tile != 0 ): # 0 is universal empty?
                    #explode
                    if selfList.count(self) > 0: # is in the list
                        selfList.remove(self)
                    break

    def getDamage(self):
        return self.damage

    def explode(self, selfList):
        self.explodeSound.play()
        selfList.remove(self)

    def draw(self, screen, camX, camY):
        self.actor.draw(screen,camX,camY, self.facingLeft, False)
