import actor, level, data, projectile, math

class Robot:
    def __init__(self, rtype):
        self.actor = actor.Actor()
        self.soundArray = {}
        self.soundArray["jump"] = data.load_sound('robotJump.wav')
        self.soundArray["shoot"] = data.load_sound('robotShoot.wav')
        if rtype == "Climb":
            self.actor.loadSprite("climber_stand1.png","stand1")
            self.actor.gravity = -0.25 # hack to mess with gravity
            self.walkSpeed = 3.0
            self.jumpSpeed = 9.0
            self.fireMax = 10 # max speed we can shoot in succession
            self.fireSpeed = 0.1 # speed of our firing!
            self.maxBullets = 1
        elif rtype == "Gravity":
            self.actor.loadSprite("gravity_stand1.png","stand1")
            self.actor.gravity = -0.25
            self.walkSpeed = 3.0
            self.jumpSpeed = 8.0
            self.hasFlipped = False # only flip once per landing
            self.soundArray["reverse"] = data.load_sound('gravityReverse.wav')
            self.fireMax = 5 # max speed we can shoot in succession
            self.fireSpeed = 5.0 # speed of our firing!
            self.maxBullets = 3
        elif rtype == "Runner":
            self.actor.loadSprite("runner_stand1.png","stand1")
            self.actor.gravity = -0.20
            self.runSpeed = 0.2
            self.maxSpeed = 11.0 # max speed
            self.jumpSpeed = 7.5
            self.fireMax = 15 # max speed we can shoot in succession
            self.fireSpeed = 8.0 # speed of our firing!
            self.maxBullets = 1
        elif rtype == "Firepower":
            self.actor.loadSprite("firepower_stand1.png","stand1")
            self.actor.gravity = -0.10
            self.walkSpeed = 2.0
            self.jumpSpeed = 4.5
            self.fireMax = 20 # max speed we can shoot in succession
            self.fireSpeed = 1.0 # speed of our firing!
            self.maxBullets = 3
            
        self.actor.setSprite("stand1")
        self.fireTrigger = False
        self.fireCnt = 0
        
        self.leftTrigger = False
        self.rightTrigger = False
        self.jumpTrigger = False
        self.roboType = rtype
        self.upTrigger = False
        self.downTrigger = False
        self.climbTrigger = False
        self.climbSpeed = 2.0
        self.facingLeft = False

    def update(self, curLevel, enemyList, projList):
        if self.roboType == "Climb":
            self.climbUpdate(curLevel, projList)
        elif self.roboType == "Gravity":
            self.gravityUpdate(curLevel, enemyList, projList)
        elif self.roboType == "Runner":
            self.runnerUpdate(curLevel, projList)
        elif self.roboType == "Firepower":
            self.fireUpdate(curLevel, projList, enemyList)

    def reset(self): # reset everything with our roboto!
        self.actor.setSprite("stand1")
        self.fireTrigger = False
        self.fireCnt = 0
        self.leftTrigger = False
        self.rightTrigger = False
        self.jumpTrigger = False
        self.upTrigger = False
        self.downTrigger = False
        self.climbTrigger = False
        self.facingLeft = False
        self.actor.reset()
    
    def fireUpdate(self, curLevel, projList,enemyList):
        if self.fireTrigger == True:
            if ( len(projList) < self.maxBullets ):
                if self.fireCnt < self.fireMax:
                    self.fireCnt += 1
                else:
                    self.fireCnt = 0
                    mindis = 9999
                    echoice = None
                    for e in enemyList:
                        newdx = abs(e.actor.px-self.actor.px)
                        newdy = abs(e.actor.py-self.actor.py)
                        if ( newdx < 400 and newdy < 100 ):
                            if math.sqrt(newdy*newdy+newdx*newdx) < mindis:
                                echoice = e
                    if echoice != None:
                        if self.facingLeft == True:
                            projList.append(projectile.Projectile("Firepower", self.actor.px-16,self.actor.py+8,-self.fireSpeed,0.0, echoice))
                        else:
                            projList.append(projectile.Projectile("Firepower", self.actor.px+32,self.actor.py+8,self.fireSpeed,0.0, echoice))
                    else:
                        if self.facingLeft == True:
                            projList.append(projectile.Projectile("Firepower", self.actor.px-16,self.actor.py+8,-self.fireSpeed,0.0))
                        else:
                            projList.append(projectile.Projectile("Firepower", self.actor.px+32,self.actor.py+8,self.fireSpeed,0.0))
        else:
            self.fireCnt = self.fireMax
        if self.leftTrigger == True:
            self.actor.setVelocityX(-self.walkSpeed)
            self.facingLeft = True
        elif self.rightTrigger == True:
            self.actor.setVelocityX(self.walkSpeed)
            self.facingLeft = False
        elif self.leftTrigger == self.rightTrigger == False:
            self.actor.setVelocityX(0.0)

        if not self.actor.inFlight():
            if self.jumpTrigger == True:
                if self.actor.upsideDown == False:
                    self.actor.jump(self.jumpSpeed) 
                else:
                    self.actor.jump(-self.jumpSpeed)
                self.soundArray["jump"].play()
        else:
            if self.jumpTrigger == False:
                self.actor.releaseJump()

        self.actor.update()
        # bm - bottom-middle section
        # lm - left-middle section
        # rm - right-middle section
        # tm - top-middle section
        # Collision detection for feet
        colX = {}
        colY = {}

        actorX,actorY = self.actor.getPos()
        # we know that the robots width & height are 32!
        colX['bm'] = int(actorX + (16))
        colY['bm'] = int(actorY + (32))
        colX['tm'] = int(actorX + (16))
        colY['tm'] = int(actorY)
        colX['lm'] = int(actorX)
        colY['lm'] = int(actorY + (16))
        colX['rm'] = int(actorX + 32)
        colY['rm'] = int(actorY + (16))

        collisionOrder = ['lm','rm','tm','bm']

        for colName in collisionOrder:
            ty = (colY[colName]/32) * curLevel.getWidth()
            tx = (colX[colName]/32)
            if self.actor.inFlight() == True:
                tile = curLevel.data[3][(ty+tx)]
                if ( tile != 0 ): # 0 is universal empty?
                    # make this universal?
                    if ( colName == 'bm' ):
                        if self.actor.upsideDown == False:
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.hitFloor()
                            self.jumpTrigger = False
                        else:
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                    elif ( colName == 'tm' ):
                        if self.actor.upsideDown == False:
                            self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                        else:
                            self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                            # -1 because we are technically moving the y down 1 more (y+32 when y is zero based!)
                            self.actor.hitFloor()
                            self.jumpTrigger = False
                    elif ( colName == 'lm' ):
                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))-1
                    elif ( colName == 'rm' ):
                        self.actor.px = self.actor.px - (colX[colName] % 32)
            elif self.actor.inFlight() == False:
                if ( curLevel.data[3][(ty+tx)] == 0 ): # 0 is universal empty?
                    if self.actor.upsideDown == False:
                        if ( colName == 'bm' ):
                            self.actor.fall()
                    else:
                        if ( colName == 'tm' ):
                            self.actor.fall()
                elif ( curLevel.data[3][(ty+tx)] != 0 ): # tile is not empty
                    if ( colName == 'lm' ):
                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))
                    elif ( colName == 'rm' ):
                        self.actor.px = self.actor.px - (colX[colName] % 32)

    def runnerUpdate(self, curLevel, projList):
        if self.fireTrigger == True:
            if ( len(projList) < self.maxBullets ):
                if self.fireCnt < self.fireMax:
                    self.fireCnt += 1
                else:
                    self.fireCnt = 0
                    #self.soundArray["shoot"].play()
                    if self.facingLeft == True:
                        projList.append(projectile.Projectile("Runner", \
                            self.actor.px-34,self.actor.py-1,-self.fireSpeed-abs(self.actor.vx),0.0))
                    else:
                        projList.append(projectile.Projectile("Runner", \
                            self.actor.px+34,self.actor.py-1,self.fireSpeed+abs(self.actor.vx),0.0))
        else:
            self.fireCnt = self.fireMax
        if self.leftTrigger == True:
            self.actor.setAccelX(-self.runSpeed)
            self.facingLeft = True
        elif self.rightTrigger == True:
            self.actor.setAccelX(self.runSpeed)
            self.facingLeft = False
        elif self.leftTrigger == self.rightTrigger == False:
            if abs(self.actor.vx) > 0.1:
                self.actor.vx = self.actor.vx * 0.90
            else:
                self.actor.vx = 0.0
        if self.actor.vx > self.maxSpeed:
            self.actor.vx = self.maxSpeed
        elif self.actor.vx < -self.maxSpeed:
            self.actor.vx = -self.maxSpeed

        if not self.actor.inFlight():
            if self.jumpTrigger == True:
                self.actor.jump(self.jumpSpeed)
                self.soundArray["jump"].play()
        else:
            if self.jumpTrigger == False:
                self.actor.releaseJump()

        self.actor.update()
        # bm - bottom-middle section
        # lm - left-middle section
        # rm - right-middle section
        # tm - top-middle section
        # Collision detection for feet
        colX = {}
        colY = {}

        actorX,actorY = self.actor.getPos()
        # we know that the robots width & height are 32!
        colX['bm'] = int(actorX + (16))
        colY['bm'] = int(actorY + (32))
        colX['tm'] = int(actorX + (16))
        colY['tm'] = int(actorY)
        colX['lm'] = int(actorX)
        colY['lm'] = int(actorY + (16))
        colX['rm'] = int(actorX + 32)
        colY['rm'] = int(actorY + (16))

        collisionOrder = ['lm','rm','tm','bm']

        for colName in collisionOrder:
            ty = (colY[colName]/32) * curLevel.getWidth()
            tx = (colX[colName]/32)
            if self.actor.inFlight() == True:
                tile = curLevel.data[3][(ty+tx)]
                if ( tile != 0 ): # 0 is universal empty?
                    # make this universal?
                    if ( colName == 'bm' ):
                        if self.actor.upsideDown == False:
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.hitFloor()
                            self.jumpTrigger = False
                        else:
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                    elif ( colName == 'tm' ):
                        if self.actor.upsideDown == False:
                            self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                        else:
                            self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                            # -1 because we are technically moving the y down 1 more (y+32 when y is zero based!)
                            self.actor.hitFloor()
                            self.jumpTrigger = False
                    elif ( colName == 'lm' ):
                        self.actor.vx = 0.0
                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))-1
                    elif ( colName == 'rm' ):
                        self.actor.vx = 0.0
                        self.actor.px = self.actor.px - (colX[colName] % 32)
            elif self.actor.inFlight() == False:
                if ( curLevel.data[3][(ty+tx)] == 0 ): # 0 is universal empty?
                    if self.actor.upsideDown == False:
                        if ( colName == 'bm' ):
                            self.actor.fall()
                    else:
                        if ( colName == 'tm' ):
                            self.actor.fall()
                elif ( curLevel.data[3][(ty+tx)] != 0 ): # tile is not empty
                    if ( colName == 'lm' ):
                        self.actor.vx = 0.0
                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))
                    elif ( colName == 'rm' ):
                        self.actor.vx = 0.0
                        self.actor.px = self.actor.px - (colX[colName] % 32)

    def gravityUpdate(self, curLevel, eList, projList):
        if self.fireTrigger == True:
            if ( len(projList) < self.maxBullets ):
                if self.fireCnt < self.fireMax:
                    self.fireCnt += 1
                else:
                    self.fireCnt = 0
                    #self.soundArray["shoot"].play()
                    if self.upTrigger == True:
                        if self.actor.upsideDown == False:
                            projList.append(projectile.Projectile("Gravity", \
                                self.actor.px+self.actor.rect.width/2,self.actor.py,0.0,-self.fireSpeed))
                        else:
                            projList.append(projectile.Projectile("Gravity", \
                                self.actor.px+self.actor.rect.width/2,self.actor.py,0.0,self.fireSpeed))
                    elif self.facingLeft == True:
                        projList.append(projectile.Projectile("Gravity", \
                            self.actor.px-8,self.actor.py+13,-self.fireSpeed,0.0))
                    else:
                        projList.append(projectile.Projectile("Gravity", \
                            self.actor.px+32,self.actor.py+13,self.fireSpeed,0.0))
        else:
            self.fireCnt = self.fireMax
        
        if self.leftTrigger == True:
            self.actor.setVelocityX(-self.walkSpeed)
            self.facingLeft = True
        elif self.rightTrigger == True:
            self.actor.setVelocityX(self.walkSpeed)
            self.facingLeft = False
        elif self.leftTrigger == self.rightTrigger == False:
            self.actor.setVelocityX(0.0)

        if self.upTrigger == True:
            if self.hasFlipped == False:
                if self.jumpTrigger == True:
                    self.hasFlipped = True
                    self.actor.airBorne = True
                    self.actor.upsideDown = not self.actor.upsideDown
                    self.soundArray["reverse"].play()
                    for e in eList:
                        e.actor.upsideDown = not e.actor.upsideDown
                        e.actor.airBorne = True
        if not self.actor.inFlight():
            if self.jumpTrigger == True:
                if self.actor.upsideDown == False:
                    self.actor.jump(self.jumpSpeed) 
                else:
                    self.actor.jump(-self.jumpSpeed)
                self.soundArray["jump"].play()
        else:
            if self.jumpTrigger == False:
                self.actor.releaseJump()

        self.actor.update()
        # bm - bottom-middle section
        # lm - left-middle section
        # rm - right-middle section
        # tm - top-middle section
        # Collision detection for feet
        colX = {}
        colY = {}

        actorX,actorY = self.actor.getPos()
        # we know that the robots width & height are 32!
        colX['bm'] = int(actorX + (16))
        colY['bm'] = int(actorY + (32))
        colX['tm'] = int(actorX + (16))
        colY['tm'] = int(actorY)
        colX['lm'] = int(actorX)
        colY['lm'] = int(actorY + (16))
        colX['rm'] = int(actorX + 32)
        colY['rm'] = int(actorY + (16))

        collisionOrder = ['lm','rm','tm','bm']

        for colName in collisionOrder:
            ty = (colY[colName]/32) * curLevel.getWidth()
            tx = (colX[colName]/32)
            if self.actor.inFlight() == True:
                tile = curLevel.data[3][(ty+tx)]
                if ( tile != 0 ): # 0 is universal empty?
                    # make this universal?
                    if ( colName == 'bm' ):
                        if self.actor.upsideDown == False:
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.hitFloor()
                            self.jumpTrigger = False
                            self.hasFlipped = False # reset our flipping condition
                        else:
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                    elif ( colName == 'tm' ):
                        if self.actor.upsideDown == False:
                            self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                        else:
                            self.actor.py = self.actor.py + (32- colY[colName] % 32)-1 # 32 is our tile height
                            # -1 because we are technically moving the y down 1 more (y+32 when y is zero based!)
                            self.actor.hitFloor()
                            self.jumpTrigger = False
                            self.hasFlipped = False # reset our flipping condition
                    elif ( colName == 'lm' ):
                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))-1
                    elif ( colName == 'rm' ):
                        self.actor.px = self.actor.px - (colX[colName] % 32)
            elif self.actor.inFlight() == False:
                if ( curLevel.data[3][(ty+tx)] == 0 ): # 0 is universal empty?
                    if self.actor.upsideDown == False:
                        if ( colName == 'bm' ):
                            self.actor.fall()
                    else:
                        if ( colName == 'tm' ):
                            self.actor.fall()
                elif ( curLevel.data[3][(ty+tx)] != 0 ): # tile is not empty
                    if ( colName == 'lm' ):
                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))
                    elif ( colName == 'rm' ):
                        self.actor.px = self.actor.px - (colX[colName] % 32)


    def climbUpdate(self, curLevel, projList):
        if self.fireTrigger == True:
            if ( len(projList) < self.maxBullets ):
                if self.fireCnt < self.fireMax:
                    self.fireCnt += 1
                else:
                    self.fireCnt = 0
                    #self.soundArray["shoot"].play()
                    if self.facingLeft == True:
                        projList.append(projectile.Projectile("Climb", \
                            self.actor.px-32,self.actor.py-1,-self.fireSpeed,0.0))
                    else:
                        projList.append(projectile.Projectile("Climb", \
                            self.actor.px+32,self.actor.py-1,self.fireSpeed,0.0))
        else:
            self.fireCnt = self.fireMax
        if self.actor.climbing == True:
            if self.jumpTrigger == True:
                self.actor.jump(self.jumpSpeed*3)
                self.actor.climbing = False
            if self.upTrigger == True:
                self.actor.climbUp(self.climbSpeed)
            elif self.downTrigger == True:
                self.actor.climbDown(self.climbSpeed)

            self.actor.update()
##            # bm - bottom-middle section
##            # lm - left-middle section
##            # rm - right-middle section
##            # tm - top-middle section
##            # Collision detection for feet
##            colX = {}
##            colY = {}
##
##            actorX,actorY = self.actor.getPos()
##            # we know that the robots width & height are 32!
##            colX['bm'] = int(actorX + (16))
##            colY['bm'] = int(actorY + (32))
##            colX['tm'] = int(actorX + (16))
##            colY['tm'] = int(actorY)
##            colX['lm'] = int(actorX)
##            colY['lm'] = int(actorY + (16))
##            colX['rm'] = int(actorX + 32)
##            colY['rm'] = int(actorY + (16))
##
##            collisionOrder = ['lm','rm','tm','bm']
##
##            for colName in collisionOrder:
##                ty = (colY[colName]/32) * curLevel.getWidth()
##                tx = (colX[colName]/32)
##                if ( curLevel.data[3][(ty+tx)] != 0 ): # tile is not empty
##                    if ( colName == 'lm' ):
##                        self.actor.px = self.actor.px + (32-(colX[colName] % 32))
##                    elif ( colName == 'rm' ):
##                        self.actor.px = self.actor.px - (colX[colName] % 32)
        else:
            if self.leftTrigger == True:
                self.actor.setVelocityX(-self.walkSpeed)
                self.facingLeft = True
            elif self.rightTrigger == True:
                self.actor.setVelocityX(self.walkSpeed)
                self.facingLeft = False
            elif self.leftTrigger == self.rightTrigger == False:
                self.actor.setVelocityX(0.0)
            if not self.actor.inFlight():
                if self.jumpTrigger == True:
                    self.actor.jump(self.jumpSpeed)
                    self.soundArray["jump"].play()
            if self.actor.inFlight():
                if self.jumpTrigger == False:
                    self.actor.releaseJump()
            
            self.actor.update()
            # bm - bottom-middle section
            # lm - left-middle section
            # rm - right-middle section
            # tm - top-middle section
            # Collision detection for feet
            colX = {}
            colY = {}

            actorX,actorY = self.actor.getPos()
            # we know that the robots width & height are 32!
            colX['bm'] = int(actorX + (16))
            colY['bm'] = int(actorY + (32))
            colX['tm'] = int(actorX + (16))
            colY['tm'] = int(actorY)
            colX['lm'] = int(actorX)
            colY['lm'] = int(actorY + (16))
            colX['rm'] = int(actorX + 32)
            colY['rm'] = int(actorY + (16))

            collisionOrder = ['lm','rm','tm','bm']

            for colName in collisionOrder:
                ty = (colY[colName]/32) * curLevel.getWidth()
                tx = (colX[colName]/32)
                if self.actor.inFlight() == True:
                    tile = curLevel.data[3][(ty+tx)]
                    if ( tile != 0 ): # 0 is universal empty?
                        # make this universal?
                        if ( colName == 'bm' ):
                            self.actor.py = self.actor.py - (colY[colName] % 32) # 32 is our tile height
                            self.actor.hitFloor()
                            self.jumpTrigger = False
                        elif ( colName == 'tm' ):
                            self.actor.py = self.actor.py + (32- colY[colName] % 32) # 32 is our tile height
                            self.actor.setVelocityY(0.0)
                        elif ( colName == 'lm' ):
                            self.actor.px = self.actor.px + (32-(colX[colName] % 32))
                            if ( tile == 4 ):
                                self.actor.climbing = True
                        elif ( colName == 'rm' ):
                            self.actor.px = self.actor.px - (colX[colName] % 32)
                            if ( tile == 3 ):
                                self.actor.climbing = True
                elif self.actor.inFlight() == False:
                    if ( curLevel.data[3][(ty+tx)] == 0 ): # 0 is universal empty?
                        if ( colName == 'bm' ):
                            self.actor.fall()
                    elif ( curLevel.data[3][(ty+tx)] != 0 ): # tile is not empty
                        if ( colName == 'lm' ):
                            self.actor.px = self.actor.px + (32-(colX[colName] % 32))
                        elif ( colName == 'rm' ):
                            self.actor.px = self.actor.px - (colX[colName] % 32)
                        
    def draw(self,screen,camX,camY):
        self.actor.draw(screen,camX,camY, self.facingLeft, self.actor.upsideDown)

    def setStart(self,x,y):
        self.actor.setPos(x,y)

    def fire(self, trigger):
        self.fireTrigger = trigger

    def moveLeft(self, trigger):
        self.leftTrigger = trigger

    def moveRight(self, trigger):
        self.rightTrigger = trigger

    def moveUp(self, trigger):
        self.upTrigger = trigger

    def moveDown(self, trigger):
        self.downTrigger = trigger

    def jump(self, trigger):
        self.jumpTrigger = trigger

    def damage(self):
        #print "ROBOT IS DAMAGED!!! ESPLODE HERE PLZ THX"
        pass
